from lib2to3.fixer_base import BaseFix
import lib2to3.pgen2
from lib2to3.pygram import python_symbols as syms
from lib2to3.pytree import Leaf, Node
import os

from my2to3.trace import get_data
from my2to3.util import add_future, data2types


def analyze_types(types):
  """Indicates whether the division described by `data` should be modified into
  `//`, or should remain as `/`
  """
  if len(types) == 1:
    dividend, divisor = types[0]
    if dividend is divisor is int:
      return True
  return False


class FixDivisionSupport(BaseFix):
  """Rewrites division_traced(n, a, b) into Py2/Py3-compatible division
  """

  # Inspired by https://github.com/python/cpython/blob/e42b705188271da108de42b55d9344642170aa2b/Lib/lib2to3/fixes/fix_xrange.py#L14
  PATTERN = """
            power<'division_traced' trailer< '(' args=any ')' >
            rest=any* >
            """

  def transform(self, node, results):
    filename, lineno, id_ = [l.value for l in node.children[1].children[1].children[:-4:2]]
    data = get_data("division", ["dividend_type", "divisor_type"], dict(filename=filename, lineno=lineno, id=id_))
    if not data:
      return

    if analyze_types(data2types(data)):
      add_future(node, 'division')
      operator = Leaf(lib2to3.pgen2.token.DOUBLESLASH, "//")
    else:
      operator = Leaf(lib2to3.pgen2.token.SLASH, "/")
    operator.prefix = node.children[1].children[1].children[3].prefix

    node.replace(
      Node(
        syms.term,
        [
          node.children[1].children[1].children[2].clone(),
          operator,
          node.children[1].children[1].children[4].clone()
        ],
        prefix=node.prefix))
